from datetime import datetime
from django.conf import settings
from provider.constants import EXPIRE_DELTA, EXPIRE_CODE_DELTA
import hashlib
import shortuuid


def short_token():
    """
    Generate a hash that can be used as an application identifier
    """
    hash = hashlib.sha1(shortuuid.uuid())
    hash.update(settings.SECRET_KEY)
    return hash.hexdigest()[::2]

def long_token():
    """
    Generate a hash that can be used as an application secret
    """
    hash = hashlib.sha1(shortuuid.uuid())
    hash.update(settings.SECRET_KEY)
    return hash.hexdigest()
    
def get_token_expiry():
    """
    Return a datetime object indicating when an access token should expire. 
    Can be customized by setting :attr:`settings.OAUTH_EXPIRE_DELTA` to a 
    :attr:`datetime.timedelta` object.
    """
    return datetime.now() + EXPIRE_DELTA

def get_code_expiry():
    """
    Return a datetime object indicating when an authorization code should 
    expire.
    Can be customized by setting :attr:`settings.OAUTH_EXPIRE_CODE_DELTA` to a 
    :attr:`datetime.timedelta` object.
    """
    return datetime.now() + EXPIRE_CODE_DELTA




# Create your views here.
from datetime import datetime, timedelta
from django.core.urlresolvers import reverse
from provider.oauth2.backends import BasicClientBackend, RequestParamsClientBackend
from provider.oauth2.forms import AuthorizationRequestForm, AuthorizationForm, \
    PasswordGrantForm, RefreshTokenGrantForm, AuthorizationCodeGrantForm
from provider.oauth2.models import Client, RefreshToken, AccessToken
from provider.views import Capture, Authorize, Redirect, \
    AccessToken as AccessTokenView, OAuthError

class Capture(Capture):
    """
    Implementation of :class:`provider.views.Capture`.
    """
    def get_redirect_url(self, request):
        return reverse('oauth2:authorize-2')
    
class Authorize(Authorize):
    """
    Implementation of :class:`provider.views.Authorize`.
    """
    def get_request_form(self, client, data):
        return AuthorizationRequestForm(data, client=client)
    
    def get_authorization_form(self, request, client, data, client_data):
        return AuthorizationForm(data)
    
    def get_client(self, client_id):
        try:
            return Client.objects.get(client_id=client_id)
        except Client.DoesNotExist:
            return None

    def get_redirect_url(self, request):
        return reverse('oauth2:redirect')

    def save_authorization(self, request, client, form, client_data):

        grant = form.save(commit=False)

        if grant is None:
            return None

        grant.user = request.user
        grant.client = client
        grant.redirect_uri = client_data.get('redirect_uri', '')
        grant.save()
        return grant.code


class Redirect(Redirect):
    """
    Implementation of :class:`provider.views.Redirect`
    """
    pass

class AccessTokenView(AccessTokenView):
    """
    Implementation of :class:`provider.views.AccessToken`.
    
    .. note:: This implementation does provide all default grant types defined
        in :attr:`provider.views.AccessToken.grant_types`. If you 
        wish to disable any, you can override the :meth:`get_handler` method
        *or* the :attr:`grant_types` list.
    """
    authentication = (
        BasicClientBackend,
        RequestParamsClientBackend,
    )
    
    def get_authorization_code_grant(self, request, data, client):
        form = AuthorizationCodeGrantForm(data, client=client)
        if not form.is_valid():
            raise OAuthError(form.errors)
        return form.cleaned_data.get('grant')

    def get_refresh_token_grant(self, request, data, client):
        form = RefreshTokenGrantForm(data, client=client)
        if not form.is_valid():
            raise OAuthError(form.errors)
        return form.cleaned_data.get('refresh_token')
    
    def get_password_grant(self, request, data, client):
        form = PasswordGrantForm(data, client=client)
        if not form.is_valid():
            raise OAuthError(form.errors)
        return form.cleaned_data
        
    def create_access_token(self, request, user, scope, client):
        return AccessToken.objects.create(
            user=user,
            client=client,
            scope=scope
        )
            
    def create_refresh_token(self, request, user, scope, access_token, client):
        return RefreshToken.objects.create(
            user=user,
            access_token=access_token,
            client=client
        )
    
    def invalidate_grant(self, grant):
        grant.expires = datetime.now() - timedelta(days=1)
        grant.save()
        
    def invalidate_refresh_token(self, rt):
        rt.expired = True
        rt.save()
    
    def invalidate_access_token(self, at):
        at.expires = datetime.now() - timedelta(days=1)
        at.save()



from datetime import datetime
from django import forms
from django.contrib.auth import authenticate
from django.utils.encoding import smart_unicode
from django.utils.translation import ugettext as _
from provider import constants
from provider.constants import RESPONSE_TYPE_CHOICES, SCOPES
from provider.forms import OAuthForm, OAuthValidationError
from provider.oauth2 import scope
from provider.oauth2.models import Client, Grant, RefreshToken
from provider.oauth2.scope import SCOPE_NAMES, SCOPE_NAME_DICT

class ClientAuthForm(forms.Form):
    """
    Client authentication form. Required to make sure that we're dealing with a
    real client. Form is used in :attr:`provider.oauth2.backends` to validate
    the client.
    """
    client_id = forms.CharField()
    client_secret = forms.CharField()
    
    def clean(self):
        data = self.cleaned_data
        try:
            client = Client.objects.get(client_id=data.get('client_id'),
                client_secret=data.get('client_secret'))
        except Client.DoesNotExist:
            raise forms.ValidationError(_("Client could not be validated with key pair."))

        data['client'] = client
        return data

class ScopeChoiceField(forms.ChoiceField):
    """ 
    Custom form field that seperates values on space as defined in :draft:`3.3`.
    """
    widget = forms.SelectMultiple
    
    def to_python(self, value):
        if not value:
            return []
        
        if not isinstance(value, (list, tuple)):
            raise OAuthValidationError({'error': 'invalid_request'})

        # Split values into list
        return u' '.join([smart_unicode(val) for val in value]).split(u' ')


    def validate(self, value):
        """
        Validates that the input is a list or tuple.
        """
        if self.required and not value:
            raise OAuthValidationError({'error': 'invalid_request'})

        # Validate that each value in the value list is in self.choices.
        for val in value:
            if not self.valid_value(val):
                raise OAuthValidationError({'error': 'invalid_request',
                    'error_description': _("'%s' is not a valid scope.") % val})
    
class ScopeMixin(object):
    """
    Form mixin to clean scope fields.
    """
    def clean_scope(self):
        """
        The scope is assembled by combining all the set flags into a single 
        integer value which we can later check again for set bits.
        
        If *no* scope is set, we return the default scope which is the first
        defined scope in :attr:`provider.constants.SCOPES`.
        
        """
        scope = SCOPES[0][0]
        
        flags = self.cleaned_data.get('scope')
        
        if not flags:
            return scope
        
        return reduce(lambda prev, next: (prev | SCOPE_NAME_DICT[next]), flags, scope)
        

class AuthorizationRequestForm(ScopeMixin, OAuthForm):
    """
    This form is used to validate the request data that the authorization 
    endpoint receives from clients.
    
    Included data is specified in :draft:`4.1.1`.
    """
    # Setting all required fields to false to explicitly check by hand
    # and use custom error messages that can be reused in the OAuth2
    # protocol
    response_type = forms.CharField(required=False)
    """
    ``"code"`` or ``"token"`` depending on the grant type.
    """
    
    redirect_uri = forms.URLField(required=False)
    """
    Where the client would like to redirect the user
    back to. This has to match whatever value was saved while creating
    the client.
    """
    
    state = forms.CharField(required=False)
    """
    Opaque - just pass back to the client for validation.
    """
    
    scope = ScopeChoiceField(choices=SCOPE_NAMES, required=False)
    """
    The scope that the authorization should include.
    """
    
    def clean_response_type(self):
        """
        :draft:`3.1.1` Lists of values are space delimited.
        """
        response_type = self.cleaned_data.get('response_type')
        
        if not response_type:
            raise OAuthValidationError({'error': 'invalid_request',
                'error_description': "No 'response_type' supplied."})

        types = response_type.split(" ")
        
        for type in types:
            if type not in RESPONSE_TYPE_CHOICES:
                raise OAuthValidationError({'error': 'unsupported_response_type',
                    'error_description': u"'%s' is not a supported response type." % type})
        
        return response_type

    def clean_redirect_uri(self):
        """
        :draft:`3.1.2` The redirect value has to match what was saved on the 
            authorization server.
        """
        redirect_uri = self.cleaned_data.get('redirect_uri')

        if redirect_uri:
            if not redirect_uri == self.client.redirect_uri:
                raise OAuthValidationError({'error': 'invalid_request',
                    'error_description': _("The requested redirect didn't match the client settings.")})
        
        return redirect_uri        
        
class AuthorizationForm(ScopeMixin, OAuthForm):
    """
    A form used to ask the resource owner for authorization of a given client.
    """
    authorize = forms.BooleanField(required=False)
    scope = ScopeChoiceField(choices=SCOPE_NAMES, required=False)    

    def save(self, **kwargs):
        authorize = self.cleaned_data.get('authorize')

        if not authorize:
            return None
        
        grant = Grant()
        grant.scope = self.cleaned_data.get('scope')
        return grant

class RefreshTokenGrantForm(ScopeMixin, OAuthForm):
    """
    Checks and returns a refresh token.
    """
    refresh_token = forms.CharField(required=False)
    scope = ScopeChoiceField(choices=SCOPE_NAMES, required=False)
    
    def clean_refresh_token(self):
        token = self.cleaned_data.get('refresh_token')

        if not token:
            raise OAuthValidationError({'error': 'invalid_request'})
        
        try:
            token = RefreshToken.objects.get(token=token,
                expired=False, client=self.client)
        except RefreshToken.DoesNotExist:
            raise OAuthValidationError({'error': 'invalid_grant'})
        
        return token
    
    def clean(self):
        """
        Make sure that the scope is less or equal to the previous scope!
        """
        data = self.cleaned_data
        
        if 'scope' in data and not scope.check(data.get('scope'), data.get('refresh_token').access_token.scope):
            raise OAuthValidationError({'error': 'invalid_scope'})
        
        return data
    
class AuthorizationCodeGrantForm(ScopeMixin, OAuthForm):
    """
    Check and return an authorization grant.
    """
    code = forms.CharField(required=False)
    scope = ScopeChoiceField(choices=SCOPE_NAMES, required=False)
    
    def clean_code(self):
        code = self.cleaned_data.get('code')
        
        if not code:
            raise OAuthValidationError({'error': 'invalid_request'})
        
        try:
            self.cleaned_data['grant'] = Grant.objects.get(
                code=code, client=self.client, expires__gt=datetime.now())
        except Grant.DoesNotExist:
            raise OAuthValidationError({'error': 'invalid_grant'})
        
        return code
    
    def clean(self):
        """
        Make sure that the scope is less or equal to the scope allowed on the
        grant! 
        """
        data = self.cleaned_data
        # Only check if we've actually got a scope in the data
        # (read: All fields have been cleaned)
        if 'scope' in data and not scope.check(data.get('scope'), data.get('grant').scope):
            raise OAuthValidationError({'error': 'invalid_scope'})
        
        return data

class PasswordGrantForm(ScopeMixin, OAuthForm):
    """
    Validate the password of a user on a password grant request.
    """
    username = forms.CharField(required=False)
    password = forms.CharField(required=False)
    scope = ScopeChoiceField(choices=SCOPE_NAMES, required=False)
        
    def clean_username(self):
        username = self.cleaned_data.get('username')

        if not username:
            raise OAuthValidationError({'error': 'invalid_request'})
        
        return username
    
    def clean_password(self):
        password = self.cleaned_data.get('password')
        
        if not password:
            raise OAuthValidationError({'error': 'invalid_request'})
        
        return password
    
    def clean(self):
        data = self.cleaned_data
        
        user = authenticate(username=data.get('username'),
            password=data.get('password'))
        
        if user is None:
            raise OAuthValidationError({'error': 'invalid_grant'})
        
        data['user'] = user
        return data
        
        



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'Client.name'
        db.add_column('oauth2_client', 'name', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'Client.name'
        db.delete_column('oauth2_client', 'name')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'oauth2.accesstoken': {
            'Meta': {'object_name': 'AccessToken'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 2, 7, 10, 40, 0, 790902)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'scope': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'token': ('django.db.models.fields.CharField', [], {'default': "'d9373f4d09149181e1c5'", 'max_length': '255'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauth2.client': {
            'Meta': {'object_name': 'Client'},
            'client_id': ('django.db.models.fields.CharField', [], {'default': "'00385d74e0b239deac42'", 'max_length': '255'}),
            'client_secret': ('django.db.models.fields.CharField', [], {'default': "'460de077126ccc746473f8b2afab6a8aef9f542a'", 'max_length': '255'}),
            'client_type': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'redirect_uri': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'oauth2.grant': {
            'Meta': {'object_name': 'Grant'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'code': ('django.db.models.fields.CharField', [], {'default': "'ae889a1d0fc0c8569b47a7037c3d2da5cb7d0d43'", 'max_length': '255'}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 2, 8, 10, 50, 0, 788579)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'redirect_uri': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'scope': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauth2.refreshtoken': {
            'Meta': {'object_name': 'RefreshToken'},
            'access_token': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'refresh_token'", 'unique': 'True', 'to': "orm['oauth2.AccessToken']"}),
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'expired': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'token': ('django.db.models.fields.CharField', [], {'default': "'50d5f3e805e02073364c4ebfc55a94328afe25bd'", 'max_length': '255'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['oauth2']



# Create your views here.
from django.http import HttpResponseBadRequest, HttpResponse, \
    HttpResponseRedirect, HttpResponseForbidden, QueryDict
from django.utils.translation import ugettext as _
from django.views.generic.base import TemplateView, View
from provider import constants
import json
import urlparse

class OAuthError(Exception):
    """
    Exception to throw inside any views defined in :attr:`provider.views`.
    
    Any :attr:`OAuthError` thrown will be signalled to the API consumer.
    
    :attr:`OAuthError` expects a dictionary as its first argument outlining the
    type of error that occured.
    
    :example:
    
    ::
    
        raise OAuthError({'error': 'invalid_request'})
    
    The different types of errors are outlined in :draft:`4.2.2.1` and :draft:`5.2`.
    
    """

class OAuthView(TemplateView):
    """ 
    Base class for any view dealing with the OAuth flow. This class overrides
    the dispatch method of :attr:`TemplateView` to add no-caching headers to
    every response as outlined in :draft:`5.1`.
    """
    
    def dispatch(self, request, *args, **kwargs):
        response = super(OAuthView, self).dispatch(request, *args, **kwargs)
        response['Cache-Control'] = 'no-store'
        response['Pragma'] = 'no-cache'
        return response

class Mixin(object):
    """
    Mixin providing common methods required in the OAuth view defined in
    :attr:`provider.views`.
    """
    def get_data(self, request, key='params'):
        """
        Return stored data from the session store.
        
        :param key: `str` The key under which the data was stored.
        """
        return request.session.get('%s:%s' % (constants.SESSION_KEY, key))
    
    def cache_data(self, request, data, key='params'):
        """
        Cache data in the session store.
        
        :param request: :attr:`django.http.HttpRequest`
        :param data: Arbitrary data to store.
        :param key: `str` The key under which to store the data.
        """
        request.session['%s:%s' % (constants.SESSION_KEY, key)] = data
    
    def clear_data(self, request):
        """
        Clear all OAuth related data from the session store.
        """
        for key in request.session.keys():
            if key.startswith(constants.SESSION_KEY):
                del request.session[key]

    def authenticate(self, request):
        """
        Authenticate a client against all the backends configured in 
        :attr:`authentication`.
        """
        for backend in self.authentication:
            client = backend().authenticate(request)
            if client is not None:
                return client
        return None

class Capture(OAuthView, Mixin):
    """
    As stated in section :draft:`3.1.2.5` this view captures all the request 
    parameters and redirects to another URL to avoid any leakage of request
    parameters to potentially harmful JavaScripts.
    
    This application assumes that whatever web-server is used as front-end will
    handle SSL transport.
        
    If you want strict enforcement of secure communication at application level, 
    set :attr:`settings.OAUTH_ENFORCE_SECURE` to ``True``.
       
    The actual implementation is required to override :meth:`get_redirect_url`.
    """
    template_name = 'provider/authorize.html'

    def get_redirect_url(self, request):
        """
        Return a redirect to a URL where the resource owner (see :draft:`1`) 
        authorizes the client (also :draft:`1`).
        
        :return: :class:`django.http.HttpResponseRedirect`
        
        """
        raise NotImplementedError
    
    def handle(self, request, data):
        self.cache_data(request, data)

        if constants.ENFORCE_SECURE and not request.is_secure():
            return self.render_to_response({'error': 'access_denied',
                'error_description': _("A secure connection is required."),
                'next': None},
                status=400)

        return HttpResponseRedirect(self.get_redirect_url(request)) 
        
        
    def get(self, request):
        return self.handle(request, request.GET)

    def post(self, request):
        return self.handle(request, request.POST)    


class Authorize(OAuthView, Mixin):
    """
    View to handle the client authorization as outlined in :draft:`4`. 
    Implementation must override a set of methods:
    
    * :attr:`get_redirect_url`
    * :attr:`get_request_form`
    * :attr:`get_authorization_form`
    * :attr:`get_client`
    * :attr:`save_authorization`
    
    :attr:`Authorize` renders the ``provider/authorize.html`` template to 
    display the authorization form.
    
    On successful authorization, it redirects the user back to the defined 
    client callback as defined in :draft:`4.1.2`.
    
    On authorization fail :attr:`Authorize` displays an error message to the
    user with a modified redirect URL to the callback including the error
    and possibly description of the error as defined in :draft:`4.1.2.1`.
    """
    template_name = 'provider/authorize.html'
    
    def get_redirect_url(self, request):
        """
        :return: ``str`` - The client URL to display in the template after 
            authorization succeeded or failed.
        """
        raise NotImplementedError
    
    def get_request_form(self, client, data):
        """
        Return a form that is capable of validating the request data captured
        by the :class:`Capture` view.
        The form must accept a keyword argument ``client``.
        """
        raise NotImplementedError
    
    def get_authorization_form(self, request, client, data, client_data):
        """
        Return a form that is capable of authorizing the client to the resource
        owner.
        
        :return: :attr:`django.forms.Form`
        """
        raise NotImplementedError
    
    def get_client(self, client_id):
        """
        Return a client object from a given client identifier. Return ``None``
        if no client is found. An error will be displayed to the resource owner
        and presented to the client upon the final redirect.
        """
        raise NotImplementedError

    def save_authorization(self, request, client, form, client_data):
        """
        Save the authorization that the user granted to the client, involving
        the creation of a time limited authorization code as outlined in
        :draft:`4.1.2`. 
        
        Should return ``None`` in case authorization is not granted.
        Should return a string representing the authorization code grant.
        
        :return: ``None``, ``str``
        """
        raise NotImplementedError

    def _validate_client(self, request, data):
        """
        :return: ``tuple`` - ``(client or False, data or error)`` 
        """
        client = self.get_client(data.get('client_id'))
        
        if client is None:
            raise OAuthError({
                'error': 'unauthorized_client',
                'error_description': _("An unauthorized client tried to access"
                    " your resources.")
            })

        form = self.get_request_form(client, data)

        if not form.is_valid():
            raise OAuthError(form.errors)

        return client, form.cleaned_data
    
    def error_response(self, request, error, **kwargs):
        """
        Return an error to be displayed to the resource owner if anything goes
        awry. Errors can include invalid clients, authorization denials and
        other edge cases such as a wrong ``redirect_uri`` in the authorization
        request.
        
        :param request: :attr:`django.http.HttpRequest`
        :param error: ``dict`` 
            The different types of errors are outlined in :draft:`4.2.2.1` 
        """
        ctx = {}
        ctx.update(error)

        # If we got a malicious redirec_uri or client_id, remove all the cached 
        # data and tell the resource owner. We will *not* redirect back to the 
        # URL.
        
        if error['error'] in ['redirect_uri', 'unauthorized_client']:
            ctx.update(next='/')
            return self.render_to_response(ctx, **kwargs)
        
        ctx.update(next=self.get_redirect_url(request))

        return self.render_to_response(ctx, **kwargs)

    def handle(self, request, post_data=None):
        data = self.get_data(request)
        
        if data is None:
            return self.error_response(request, {'expired_authorization': True})
        
        try:
            client, data = self._validate_client(request, data)
        except OAuthError, e:
            return self.error_response(request, e.args[0], status=400)

        authorization_form = self.get_authorization_form(request, client, post_data,
            data)

        if not authorization_form.is_bound:
            return self.render_to_response({'form': authorization_form})
        
        if not authorization_form.is_valid():
            return self.render_to_response({'form': authorization_form})
        
        code = self.save_authorization(request, client, authorization_form, data)

        self.cache_data(request, data)
        self.cache_data(request, code, "code")
        self.cache_data(request, client, "client")
        
        return HttpResponseRedirect(self.get_redirect_url(request))

    def get(self, request):
        return self.handle(request, None)        
        
    def post(self, request):
        return self.handle(request, request.POST)
        
      
class Redirect(OAuthView, Mixin):
    """
    Redirect the user back to the client with the right query parameters set.
    This can be either parameters indicating success or parameters indicating
    an error.
    """
    def get(self, request):
        data = self.get_data(request)
        code = self.get_data(request, "code")
        error = self.get_data(request, "error")
        client = self.get_data(request, "client")

        self.clear_data(request)

        redirect_uri = data.get('redirect_uri', client.redirect_uri)

        parsed = urlparse.urlparse(redirect_uri)

        query = QueryDict('', mutable=True)
        
        if 'state' in data:
            query['state'] = data['state']
        
        if error is not None:
            query.update(error)
        elif code is None:
            query['error'] = 'access_denied'
        else:
            query['code'] = code
        
        parsed = parsed[:4] + (query.urlencode(), '')

        redirect_uri = urlparse.ParseResult(*parsed).geturl()
                
        return HttpResponseRedirect(redirect_uri)        

class AccessToken(OAuthView, Mixin):    
    """
    :attr:`AccessToken` handles creation and refreshing of access tokens.
    
    Implementations must implement a number of methods:
    
    * :attr:`get_authorization_code_grant`
    * :attr:`get_refresh_token_grant`
    * :attr:`get_password_grant`
    * :attr:`create_access_token`
    * :attr:`create_refresh_token`
    * :attr:`invalidate_grant`
    * :attr:`invalidate_access_token`
    * :attr:`invalidate_refresh_token`
    
    The default implementation supports the grant types defined in :attr:`grant_types`.
    
    According to :draft:`4.4.2` this endpoint too must support secure communication.
    For strict enforcement of secure communication at application level set
    :attr:`settings.OAUTH_ENFORCE_SECURE` to ``True``.
    
    According to :draft:`3.2` we can only accept POST requests.
    
    Returns with a status code of *400* in case of errors. *200* in case of 
    success.
    """
    
    authentication = ()
    """
    Authentication backends used to authenticate a particular client.
    """
    
    grant_types = ['authorization_code', 'refresh_token', 'password']
    """
    The default grant types supported by this view.
    """
    
    def get_authorization_code_grant(self, request, data, client):
        """
        Return the grant associated with this request or an error dict.
        
        :return: ``tuple`` - ``(True or False, grant or error_dict)``
        """
        raise NotImplementedError
    
    def get_refresh_token_grant(self, request, data, client): 
        """
        Return the refresh token associated with this request or an error dict.
        
        :return: ``tuple`` - ``(True or False, token or error_dict)``
        """
        raise NotImplementedError
    
    def get_password_grant(self, request, data, client): 
        """
        Return a user associated with this request or an error dict.
        
        :return: ``tuple`` - ``(True or False, user or error_dict)``
        """
        raise NotImplementedError
           
    def create_access_token(self, request, user, scope, client): 
        """
        Override to handle access token creation.
        
        :return: ``object`` - Access token
        """
        raise NotImplementedError
    
    def create_refresh_token(self, request, user, scope, access_token, client): 
        """
        Override to handle refresh token creation.
        
        :return: ``object`` - Refresh token
        """
        raise NotImplementedError
    
    def invalidate_grant(self, grant):
        """
        Override to handle grant invalidation. A grant is invalidated right after
        creating an access token from it.
        
        :return None:
        """
        raise NotImplementedError
    
    def invalidate_refresh_token(self, refresh_token):
        """
        Override to handle refresh token invalidation. When requesting a new
        access token from a refresh token, the old one is *always* invalidated.
        
        :return None:
        """
        raise NotImplementedError
    
    def invalidate_access_token(self, access_token):
        """
        Override to handle access token invalidation. When a new access token
        is created from a refresh token, the old one is *always* invalidated.
        
        :return None:
        """
        raise NotImplementedError
    
    def error_response(self, error, mimetype='application/json', status=400, **kwargs):
        """
        Return an error response to the client with default status code of *400*
        stating the error as outlined in :draft:`5.2`.
        """
        return HttpResponse(json.dumps(error), mimetype=mimetype, status=status, **kwargs)        
    
    def access_token_response(self, access_token):
        """
        Returns a successful response after creating the access token 
        as defined in :draft:`5.1`.
        """
        return HttpResponse(
            json.dumps({
                'access_token': access_token.token,
                'expires_in': access_token.get_expire_delta(),
                'refresh_token': access_token.refresh_token.token,
                'scope': access_token.scope
            }), mimetype='application/json'
        )
    
    def authorization_code(self, request, data, client):
        """
        Handle ``grant_type=authorization_token`` requests as defined in 
        :draft:`4.1.3`.
        """
        grant = self.get_authorization_code_grant(request, request.POST, client)
        
        at = self.create_access_token(request, grant.user, grant.scope, client)
        rt = self.create_refresh_token(request, grant.user, grant.scope, at, client)
        
        self.invalidate_grant(grant)
        
        return self.access_token_response(at)        
        
        
    def refresh_token(self, request, data, client):
        """
        Handle ``grant_type=refresh_token`` requests as defined in :draft:`6`.
        """
        rt = self.get_refresh_token_grant(request, data, client)
        
        self.invalidate_refresh_token(rt)
        self.invalidate_access_token(rt.access_token)
        
        at = self.create_access_token(request, rt.user, rt.access_token.scope, client)
        rt = self.create_refresh_token(request, at.user, at.scope, at, client)
        
        return self.access_token_response(at)
    
    def password(self, request, data, client):
        """
        Handle ``grant_type=password`` requests as defined in :draft:`4.3`.
        """
        
        data = self.get_password_grant(request, data, client)
        
        at = self.create_access_token(request, data.get('user'), data.get('scope'), client)
        rt = self.create_refresh_token(request, data.get('user'), data.get('scope'), at, client)
        
        return self.access_token_response(at)
        
    def get_handler(self, grant_type):
        """
        Return a function or method that is capable handling the ``grant_type``
        requested by the client or return ``None`` to indicate that this type
        of grant type is not supported, resulting in an error response.
        """
        if grant_type == 'authorization_code':
            return self.authorization_code
        elif grant_type == 'refresh_token':
            return self.refresh_token
        elif grant_type == 'password':
            return self.password
        return None
    
    def get(self, request):
        """
        As per :draft:`3.2` the token endpoint *only* supports POST requests.
        Returns an error response.
        """
        return self.error_response({'error': 'invalid_request',
            'error_description': _("Only POST requests allowed.")})
    
    def post(self, request):
        """
        As per :draft:`3.2` the token endpoint *only* supports POST requests.
        """
        if constants.ENFORCE_SECURE and not request.is_secure():
            return self.error_response({'error': 'invalid_request',
                'error_description': _("A secure connection is required.")})

        if not 'grant_type' in request.POST:
            return self.error_response({'error': 'invalid_request',
                'error_description': _("No 'grant_type' included in the request.")})
            
        grant_type = request.POST['grant_type']
        
        if grant_type not in self.grant_types:
            return self.error_response({'error': 'unsupported_grant_type'})
    
        client = self.authenticate(request)
        
        if client is None:
            return self.error_response({'error': 'invalid_client'})
        
        handler = self.get_handler(grant_type)
        
        try:
            return handler(request, request.POST, client)
        except OAuthError, e:
            return self.error_response(e.args[0])
        
        



from django import forms

class OAuthValidationError(Exception):
    """
    Exception to throw inside :class:`OAuthForm` if any OAuth2 related errors
    are encountered such as invalid grant type, invalid client, etc.
    
    :attr:`OAuthValidationError` expects a dictionary outlining the OAuth error 
    as its first argument when instantiating.
    
    :example:
    
    ::
    
        class GrantValidationForm(OAuthForm):
            grant_type = forms.CharField()
            
            def clean_grant(self):
                if not self.cleaned_data.get('grant_type') == 'code':
                    raise OAuthValidationError({
                        'error': 'invalid_grant', 
                        'error_description': "%s is not a valid grant type" % (
                            self.cleaned_data.get('grant_type'))
                    })

    The different types of errors are outlined in :draft:`4.2.2.1` and :draft:`5.2`.
    """

class OAuthForm(forms.Form):
    """
    Form class that creates shallow error dicts and exists early when a
    :class:`OAuthValidationError` is raised. 
    
    The shallow error dict is reused when returning error responses to the client.
    
    The different types of errors are outlined in :draft:`4.2.2.1` and :draft:`5.2`.
    """
    def __init__(self, *args, **kwargs):
        self.client = kwargs.pop('client', None)
        super(OAuthForm, self).__init__(*args, **kwargs)
        
    def _clean_fields(self):
        """
        Overriding the default cleaning behaviour to exit early on errors instead
        of validating each field.        
        """
        try:
            super(OAuthForm, self)._clean_fields()
        except OAuthValidationError, e:
            self._errors.update(e.args[0])

    def _clean_form(self):
        """
        Overriding the default cleaning behaviour for a shallow error dict.
        """
        try:
            super(OAuthForm, self)._clean_form()
        except OAuthValidationError, e:
            self._errors.update(e.args[0])



from django.contrib import admin
from provider.oauth2.models import AccessToken, Grant, Client, \
    RefreshToken
    
class AccessTokenAdmin(admin.ModelAdmin):
    list_display = ('user', 'client', 'token', 'expires', 'scope',)
    raw_id_fields = ('user',)

class GrantAdmin(admin.ModelAdmin):
    list_display = ('user', 'client', 'code', 'expires',)
    raw_id_fields = ('user',)
    
class ClientAdmin(admin.ModelAdmin):    
    list_display = ('url', 'user', 'redirect_uri', 'client_id', 'client_type')
    raw_id_fields = ('user',)

admin.site.register(AccessToken, AccessTokenAdmin)
admin.site.register(Grant, GrantAdmin)
admin.site.register(Client, ClientAdmin)
admin.site.register(RefreshToken)



"""
The default implementation of the OAuth provider includes two public endpoints
that are meant for client (as defined in :draft:`1`) interaction. 

.. attribute:: ^authorize/$

    This is the URL where a client should redirect a user to for authorization.
    
    This endpoint expects the parameters defined in :draft:`4.1.1` and returns
    responses as defined in :draft:`4.1.2` and :draft:`4.1.2.1`.

.. attribute:: ^access_token/$

    This is the URL where a client exchanges a grant for an access tokens. 
    
    This endpoint expects different parameters depending on the grant type:
    
    * Access tokens: :draft:`4.1.3`
    * Refresh tokens: :draft:`6`
    * Password grant: :draft:`4.3.2`
    
    This endpoint returns responses depending on the grant type:
    
    * Access tokens: :draft:`4.1.4` and :draft:`5.1`
    * Refresh tokens: :draft:`4.1.4` and :draft:`5.1`
    * Password grant: :draft:`5.1`
    
    To override, remove or add grant types, override the appropriate methods on
    :class:`provider.views.AccessToken` and / or :class:`provider.oauth2.views.AccessTokenView`.
    
    Errors are outlined in :draft:`5.2`.
    
"""

from django.conf.urls.defaults import patterns, url
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from provider.oauth2.views import Authorize, Redirect, Capture, AccessTokenView


urlpatterns = patterns('',
    url('^authorize/$', login_required(Capture.as_view()), name='authorize'),
    url('^authorize/2/$', login_required(Authorize.as_view()), name='authorize-2'),
    url('^redirect/$', login_required(Redirect.as_view()), name='redirect'),
    url('^access_token/$', csrf_exempt(AccessTokenView.as_view()), name='access_token'),
)



"""
Default model implementations. Custom database or OAuth backends need to implement 
these models with fields and and methods to be compatible with the views in
:attr:`provider.views`.
"""

from datetime import datetime
from django.contrib.auth.models import User
from django.db import models
from provider.constants import CLIENT_TYPES, SCOPES
from provider.oauth2.managers import AccessTokenManager
from provider.utils import short_token, long_token, get_token_expiry, \
    get_code_expiry


class Client(models.Model):
    """
    Default client implementation.
    
    Expected fields:
    
    * :attr:`user`
    * :attr:`name`
    * :attr:`url`
    * :attr:`redirect_url`
    * :attr:`client_id`
    * :attr:`client_secret`
    * :attr:`client_type` 
    
    Clients are outlined in the :draft:`2` and its subsections.
    """
    user = models.ForeignKey(User, blank=True, null=True)
    name = models.CharField(max_length=255, blank=True)
    url = models.URLField(help_text="Your application's URL.")
    redirect_uri = models.URLField(help_text="Your application's callback URL")
    client_id = models.CharField(max_length=255, default=short_token)
    client_secret = models.CharField(max_length=255, default=long_token)
    client_type = models.IntegerField(choices=CLIENT_TYPES)
    
    def __unicode__(self):
        return self.redirect_uri

class Grant(models.Model):
    """
    Default grant implementation. A grant is a code that can be swapped for an
    access token. Grants have a limited lifetime as defined by 
    :attr:`provider.constants.EXPIRE_CODE_DELTA` and outlined in 
    :draft:`4.1.2`
    
    Expected fields:
    
    * :attr:`user`
    * :attr:`client` - :class:`Client`
    * :attr:`code`
    * :attr:`expires` - :attr:`datetime.datetime`
    * :attr:`redirect_uri`
    * :attr:`scope`
    """
    user = models.ForeignKey(User)
    client = models.ForeignKey(Client)
    code = models.CharField(max_length=255, default=long_token)
    expires = models.DateTimeField(default=get_code_expiry)
    redirect_uri = models.CharField(max_length=255, blank=True)
    scope = models.IntegerField(default=0)
    
    def __unicode__(self):
        return self.code
    
class AccessToken(models.Model):
    """
    Default access token implementation. An access token is a time limited token
    to access a user's resources.
    
    Access tokens are outlined :draft:`5`.
    
    Expected fields:
    
    * :attr:`user`
    * :attr:`token`
    * :attr:`client` - :class:`Client`
    * :attr:`expires` - :attr:`datetime.datetime`
    * :attr:`scope`
    
    Expected methods:

    * :meth:`get_expire_delta` - returns an integer representing seconds to
        expiry
    """
    user = models.ForeignKey(User)
    token = models.CharField(max_length=255, default=short_token)
    client = models.ForeignKey(Client)
    expires = models.DateTimeField(default=get_token_expiry)
    scope = models.IntegerField(default=0)

    objects = AccessTokenManager()
    
    def __unicode__(self):
        return self.token
    
    def get_expire_delta(self):
        """
        Return the number of seconds until this token expires.
        """
        return (self.expires - datetime.now()).seconds

class RefreshToken(models.Model):
    """
    Default refresh token implementation. A refresh token can be swapped for a
    new access token when said token expires.
    
    Expected fields:
    
    * :attr:`user`
    * :attr:`token`
    * :attr:`access_token` - :class:`AccessToken`
    * :attr:`client` - :class:`Client`
    * :attr:`expired` - ``boolean``
    """
    user = models.ForeignKey(User)
    token = models.CharField(max_length=255, default=long_token)
    access_token = models.OneToOneField(AccessToken, related_name='refresh_token')
    client = models.ForeignKey(Client)
    expired = models.BooleanField(default=False)
    
    def __unicode__(self):
        return self.token



from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from provider.oauth2.models import Client, Grant
from provider.testcases import AuthorizationTest, AccessTokenTest, \
    EnforceSecureTest


class Mixin(object):
    def login(self):
        self.client.login(username='test-user-1', password='test')
    def auth_url(self):
        return reverse('oauth2:authorize')
    def auth_url2(self):
        return reverse('oauth2:authorize-2')
    def redirect_url(self):
        return reverse('oauth2:redirect')
    def access_token_url(self):
        return reverse('oauth2:access_token')
    def get_client(self):
        return Client.objects.get(id=2)
    def get_grant(self):
        return Grant.objects.all()[0]
    def get_user(self):
        return User.objects.get(id=1)
    def get_password(self):
        return 'test'
        
class AuthorizationTest(AuthorizationTest, Mixin):
    pass

class AccessTokenTest(AccessTokenTest, Mixin):
    pass

class EnforceSecureTest(EnforceSecureTest, Mixin):
    pass



from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^oauth2/', include('provider.oauth2.urls', namespace = 'oauth2')),
)



"""
Custom Sphinx documentation module to link to parts of the OAuth2 draft.
"""
from docutils import nodes, utils

base_url = "http://tools.ietf.org/html/draft-ietf-oauth-v2-23"

def rfclink(name, rawtext, text, lineno, inliner, options={}, content=[]):
    """Link to the OAuth2 draft.

    Returns 2 part tuple containing list of nodes to insert into the
    document and a list of system messages.  Both are allowed to be
    empty.

    :param name: The role name used in the document.
    :param rawtext: The entire markup snippet, with role.
    :param text: The text marked with the role.
    :param lineno: The line number where rawtext appears in the input.
    :param inliner: The inliner instance that called us.
    :param options: Directive options for customization.
    :param content: The directive content for customization.
    """

    node = nodes.reference(rawtext, "Draft Section " + text, refuri="%s#section-%s" % (base_url, text))
    
    return [node], []

def setup(app):
    """
    Install the plugin.
    
    :param app: Sphinx application context.
    """
    app.add_role('draft', rfclink)
    return



from django.conf import settings
from django.http import QueryDict
from django.test import TestCase
from django.utils.html import escape
from provider import constants
import json
import urlparse

class TestMixin(object):
    
    def _login_and_authorize(self, url_func=None):
        if url_func is None:
            url_func = lambda: self.auth_url() + '?client_id=%s&response_type=code&state=abc' % self.get_client().client_id
        
        response = self.client.get(url_func())
        response = self.client.get(self.auth_url2())
        
        response = self.client.post(self.auth_url2(), {'authorize': True, 'scope': constants.SCOPES[0][1]})
        self.assertEqual(302, response.status_code, response.content)
        self.assertTrue(self.redirect_url() in response['Location'])
        
    
class AuthorizationTest(TestCase, TestMixin):
    fixtures = ['test_oauth2']

    def setUp(self):
        self._old_login = settings.LOGIN_URL
        settings.LOGIN_URL = '/login/'
    
    def tearDown(self):
        settings.LOGIN_URL = self._old_login

    def test_authorization_requires_login(self):
        response = self.client.get(self.auth_url())
        
        # Login redirect
        self.assertEqual(302, response.status_code)
        self.assertEqual('/login/', urlparse.urlparse(response['Location']).path)

        self.login()

        response = self.client.get(self.auth_url())
        
        self.assertEqual(302, response.status_code)

        self.assertTrue(self.auth_url2() in response['Location'])
        
    def test_authorization_requires_client_id(self):
        self.login()
        response = self.client.get(self.auth_url())
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue("An unauthorized client tried to access your resources." in response.content)

    def test_authorization_rejects_invalid_client_id(self):
        self.login()
        response = self.client.get(self.auth_url() + '?client_id=123')
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue("An unauthorized client tried to access your resources." in response.content)

    def test_authorization_requires_response_type(self):
        self.login()
        response = self.client.get(self.auth_url() + '?client_id=%s' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue(escape(u"No 'response_type' supplied.") in response.content)
        
    def test_authorization_requires_supported_response_type(self):
        self.login()
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=unsupported' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())

        self.assertEqual(400, response.status_code)
        self.assertTrue(escape(u"'unsupported' is not a supported response type.") in response.content)

        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())
        self.assertEqual(200, response.status_code, response.content)
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=token' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())
        self.assertEqual(200, response.status_code)
        
    def test_authorization_requires_a_valid_redirect_uri(self):
        self.login()
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code&redirect_uri=%s' % (
            self.get_client().client_id,
            self.get_client().redirect_uri + '-invalid'))
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue(escape(u"The requested redirect didn't match the client settings.") in response.content)
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code&redirect_uri=%s' % (
            self.get_client().client_id,
            self.get_client().redirect_uri))
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(200, response.status_code)
        
    def test_authorization_requires_a_valid_scope(self):
        self.login()
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code&scope=invalid+invalid2' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue(escape(u"'invalid' is not a valid scope.") in response.content)
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code&scope=%s' % (
            self.get_client().client_id,
            constants.SCOPES[0][1]))
        response = self.client.get(self.auth_url2())
        self.assertEqual(200, response.status_code)

    def test_authorization_is_not_granted(self):
        self.login()
        
        response = self.client.get(self.auth_url() + '?client_id=%s&response_type=code' % self.get_client().client_id)
        response = self.client.get(self.auth_url2())
        
        response = self.client.post(self.auth_url2(), {'authorize': False, 'scope': constants.SCOPES[0][1]})
        self.assertEqual(302, response.status_code, response.content)
        self.assertTrue(self.redirect_url() in response['Location'])
        
        response = self.client.get(self.redirect_url())
        
        self.assertEqual(302, response.status_code)
        self.assertTrue('error=access_denied' in response['Location'])
        self.assertFalse('code' in response['Location'])

    def test_authorization_is_granted(self):
        self.login()

        self._login_and_authorize()
        
        response = self.client.get(self.redirect_url())
        
        self.assertEqual(302, response.status_code)
        self.assertFalse('error' in response['Location'])
        self.assertTrue('code' in response['Location'])
        
    def test_preserving_the_state_variable(self):
        self.login()
        
        self._login_and_authorize()
        
        response = self.client.get(self.redirect_url())
        
        self.assertEqual(302, response.status_code)
        self.assertFalse('error' in response['Location'])
        self.assertTrue('code' in response['Location'])
        self.assertTrue('state=abc' in response['Location'])

class AccessTokenTest(TestCase, TestMixin):
    fixtures = ['test_oauth2']
    
    def test_fetching_access_token_with_invalid_client(self):
        self.login()
        self._login_and_authorize()
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'authorization_code',
            'client_id': self.get_client().client_id + '123',
            'client_secret': self.get_client().client_secret, })
        
        self.assertEqual(400, response.status_code, response.content)
        self.assertEqual('invalid_client', json.loads(response.content)['error'])
        
    def test_fetching_access_token_with_invalid_grant(self):
        self.login()
        self._login_and_authorize()
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'authorization_code',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'code': '123'})
        
        self.assertEqual(400, response.status_code, response.content)
        self.assertEqual('invalid_grant', json.loads(response.content)['error'])

    def _login_authorize_get_token(self):
        self.login()
        self._login_and_authorize()
        
        response = self.client.get(self.redirect_url())
        query = QueryDict(urlparse.urlparse(response['Location']).query)
        code = query['code']
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'authorization_code',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'code': code})
        
        self.assertEqual(200, response.status_code, response.content)
        
        return json.loads(response.content)
    
    def test_fetching_access_token_with_valid_grant(self):
        self._login_authorize_get_token()        
        
    def test_fetching_access_token_with_invalid_grant_type(self):
        self.login()
        self._login_and_authorize()
        response = self.client.get(self.redirect_url())
        
        query = QueryDict(urlparse.urlparse(response['Location']).query)
        code = query['code']
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'invalid_grant_type',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'code': code
        })
        
        self.assertEqual(400, response.status_code)
        self.assertEqual('unsupported_grant_type', json.loads(response.content)['error'],
            response.content)
    
    def test_fetching_access_token_multiple_times(self):
        self._login_authorize_get_token()
        code = self.get_grant().code
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'authorization_code',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'code': code})
        
        self.assertEqual(400, response.status_code)
        self.assertEqual('invalid_grant', json.loads(response.content)['error'])
        
    def test_escalating_the_scope(self):
        self.login()
        self._login_and_authorize()
        code = self.get_grant().code
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'authorization_code',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'code': code,
            'scope': 'read write'})
        
        self.assertEqual(400, response.status_code)
        self.assertEqual('invalid_scope', json.loads(response.content)['error'])
        
    def test_refreshing_an_access_token(self):
        token = self._login_authorize_get_token()
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'refresh_token',
            'refresh_token': token['refresh_token'],
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
        })
        
        self.assertEqual(200, response.status_code)

        response = self.client.post(self.access_token_url(), {
            'grant_type': 'refresh_token',
            'refresh_token': token['refresh_token'],
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
        })
        
        self.assertEqual(400, response.status_code)
        self.assertEqual('invalid_grant', json.loads(response.content)['error'],
            response.content)
    
    def test_password_grant(self):
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'password',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'username': self.get_user().username,
            'password': self.get_password(),
        })
        
        self.assertEqual(200, response.status_code, response.content)
        
        response = self.client.post(self.access_token_url(), {
            'grant_type': 'password',
            'client_id': self.get_client().client_id,
            'client_secret': self.get_client().client_secret,
            'username': self.get_user().username,
            'password': self.get_password() + 'invalid',
        })
        
        self.assertEqual(400, response.status_code, response.content)
        self.assertEqual('invalid_grant', json.loads(response.content)['error'])
        

class EnforceSecureTest(TestCase):
    fixtures = ['test_oauth2']

    def setUp(self):
        constants.ENFORCE_SECURE = True 
    
    def tearDown(self):
        constants.ENFORCE_SECURE = False
        
    def test_authorization_enforces_SSL(self):
        self.login()

        response = self.client.get(self.auth_url())
        
        self.assertEqual(400, response.status_code)
        self.assertTrue("A secure connection is required." in response.content)
        
    def test_access_token_enforces_SSL(self):
        response = self.client.post(self.access_token_url(), {})

        self.assertEqual(400, response.status_code)
        self.assertTrue("A secure connection is required." in response.content)
        



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Client'
        db.create_table('oauth2_client', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('redirect_uri', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('client_id', self.gf('django.db.models.fields.CharField')(default='37b581bdc702c732aa65', max_length=255)),
            ('client_secret', self.gf('django.db.models.fields.CharField')(default='5cf90561f7566aa81457f8a32187dcb8147c7b73', max_length=255)),
            ('client_type', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('oauth2', ['Client'])

        # Adding model 'Grant'
        db.create_table('oauth2_grant', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauth2.Client'])),
            ('code', self.gf('django.db.models.fields.CharField')(default='f0cda1a5f4ae915431ff93f477c012b38e2429c4', max_length=255)),
            ('expires', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2012, 2, 8, 10, 43, 45, 620301))),
            ('redirect_uri', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('scope', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('oauth2', ['Grant'])

        # Adding model 'AccessToken'
        db.create_table('oauth2_accesstoken', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('token', self.gf('django.db.models.fields.CharField')(default='b10b8f721e95117cb13c', max_length=255)),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauth2.Client'])),
            ('expires', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2013, 2, 7, 10, 33, 45, 618854))),
            ('scope', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('oauth2', ['AccessToken'])

        # Adding model 'RefreshToken'
        db.create_table('oauth2_refreshtoken', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('token', self.gf('django.db.models.fields.CharField')(default='84035a870dab7c820c2c501fb0b10f86fdf7a3fe', max_length=255)),
            ('access_token', self.gf('django.db.models.fields.related.OneToOneField')(related_name='refresh_token', unique=True, to=orm['oauth2.AccessToken'])),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauth2.Client'])),
            ('expired', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('oauth2', ['RefreshToken'])


    def backwards(self, orm):
        
        # Deleting model 'Client'
        db.delete_table('oauth2_client')

        # Deleting model 'Grant'
        db.delete_table('oauth2_grant')

        # Deleting model 'AccessToken'
        db.delete_table('oauth2_accesstoken')

        # Deleting model 'RefreshToken'
        db.delete_table('oauth2_refreshtoken')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'oauth2.accesstoken': {
            'Meta': {'object_name': 'AccessToken'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 2, 7, 10, 33, 45, 624553)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'scope': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'token': ('django.db.models.fields.CharField', [], {'default': "'d5c1f65020ebdc89f20c'", 'max_length': '255'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauth2.client': {
            'Meta': {'object_name': 'Client'},
            'client_id': ('django.db.models.fields.CharField', [], {'default': "'306fb26cbcc87dd33cdb'", 'max_length': '255'}),
            'client_secret': ('django.db.models.fields.CharField', [], {'default': "'7e5785add4898448d53767f15373636b918cf0e3'", 'max_length': '255'}),
            'client_type': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'redirect_uri': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauth2.grant': {
            'Meta': {'object_name': 'Grant'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'code': ('django.db.models.fields.CharField', [], {'default': "'310b2c63e27306ecf5307569dd62340cc4994b73'", 'max_length': '255'}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 2, 8, 10, 43, 45, 625956)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'redirect_uri': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'scope': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauth2.refreshtoken': {
            'Meta': {'object_name': 'RefreshToken'},
            'access_token': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'refresh_token'", 'unique': 'True', 'to': "orm['oauth2.AccessToken']"}),
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauth2.Client']"}),
            'expired': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'token': ('django.db.models.fields.CharField', [], {'default': "'ef0ab76037f17769ab2975a816e8f41a1c11d25e'", 'max_length': '255'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['oauth2']



#!/usr/bin/env python

from setuptools import setup, find_packages
import provider

setup(
    name='django-oauth2-provider',
    version=provider.__version__,
    description='Provide OAuth2 access to your app',
    long_description=open('README.rst').read(),
    author='Alen Mujezinovic',
    author_email='alen@caffeinehit.com',
    url = 'https://github.com/caffeinehit/django-oauth2-provider',
    packages= find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=[
        "shortuuid>=0.1"
    ],
    include_package_data=True,
    zip_safe=False,
)



"""
Default scope implementation relying on bit shifting. See 
:attr:`provider.constants.SCOPES` for the list of available scopes.

Scopes can be combined, such as ``"read write"``. Note that a single ``"write"``
scope is *not* the same as ``"read write"``.

See :class:`provider.oauth2.forms.ScopeMixin` on how scopes are
combined.
"""

from provider.constants import SCOPES

SCOPE_NAMES = [(name, name) for (value, name) in SCOPES]
SCOPE_NAME_DICT = dict([(name, value) for (value, name) in SCOPES])
SCOPE_VALUE_DICT = dict([(value, name) for (value, name) in SCOPES])

def check(wants, has):
    """
    Check if a desired scope ``wants`` is part of an available scope ``has``.
    
    Returns ``False`` if not, return ``True`` if yes.
    
    :example:
        
    If a list of scopes such as
    
    ::
    
        READ = 1 << 1
        WRITE = 1 << 2
        READ_WRITE = READ | WRITE
    
        SCOPES = (
            (READ, 'read'),
            (WRITE, 'write'),
            (READ_WRITE, 'read+write'),
        )
    
    is defined, we can check if a given scope is part of another:
    
    ::
    
        >>> from provider.oauth2 import scope
        >>> scope.check(READ, READ)
        True
        >>> scope.check(WRITE, READ)
        False
        >>> scope.check(WRITE, WRITE)
        True
        >>> scope.check(READ, WRITE)
        False
        >>> scope.check(READ, READ_WRITE)
        True
        >>> scope.check(WRITE, READ_WRITE)
        True        
    
    """
    if wants & has == 0:
        return False
    if wants & has < wants:
        return False
    return True



from datetime import datetime
from provider.oauth2.forms import ClientAuthForm
from provider.oauth2.models import Client, AccessToken

class BaseBackend(object):
    """
    Base backend used to authenticate clients as defined in :draft:`1` against
    our database.
    """
    def authenticate(self, request=None):
        """
        Override this method to implement your own authentication backend.
        Return a client or ``None`` in case of failure.
        """

class BasicClientBackend(object):
    """
    Backend that tries to authenticate a client through HTTP authorization headers
    as defined in :draft:`2.3.1`.
    """
    def authenticate(self, request=None):
        auth = request.META.get('HTTP_AUTHORIZATION')

        if auth is None or auth == '':
            return None
        
        try:
            basic, base64 = auth.split(' ')
            client_id, client_secret = base64.decode('base64').split(':')
            
            form = ClientAuthForm({'client_id': client_id, 'client_secret': client_secret})
            
            if form.is_valid():
                return form.cleaned_data.get('client')
            return None

        except ValueError, e:
            # Auth header was malformed, unpacking went wrong
            return None


class RequestParamsClientBackend(object):
    """
    Backend that tries to authenticate a client through request parameters
    which might be in the request body or URI as defined in :draft:`2.3.1`.
    """
    def authenticate(self, request=None):
        if request is None:
            return None
        
        form = ClientAuthForm(request.REQUEST)
        
        if form.is_valid():
            return form.cleaned_data.get('client')
        
        return None
        
        
class AccessTokenBackend(object):
    """ 
    Authenticate a user via access token and client object.
    """
    
    def authenticate(self, access_token=None, client=None):
        try:
            return AccessToken.objects.get(access_token=access_token,
                expiry__gt=datetime.now(), client=client)
        except AccessToken.DoesNotExist:
            return None




from datetime import timedelta
from django.conf import settings

CONFIDENTIAL = 0
PUBLIC = 1

CLIENT_TYPES = (
    (CONFIDENTIAL, "Confidential (Web applications)"),
    (PUBLIC, "Public (Native and JS applications)")
)

RESPONSE_TYPE_CHOICES = getattr(settings, 'OAUTH_RESPONSE_TYPE_CHOICES', ("code", "token"))

READ = 1 << 1
WRITE = 1 << 2  

DEFAULT_SCOPES = (
    (READ, 'read'),
    (WRITE, 'write'),
)

SCOPES = getattr(settings, 'OAUTH_SCOPES', DEFAULT_SCOPES)

EXPIRE_DELTA = getattr(settings, 'OAUTH_EXPIRE_DELTA', timedelta(days=365))

EXPIRE_CODE_DELTA = getattr(settings, 'OAUTH_EXPIRE_CODE_DELTA', timedelta(seconds=10 * 60))

ENFORCE_SECURE = getattr(settings, 'OAUTH_ENFORCE_SECURE', False)
ENFORCE_CLIENT_SECURE = getattr(settings, 'OAUTH_ENFORCE_CLIENT_SECURE', True)

SESSION_KEY = getattr(settings, 'OAUTH_SESSION_KEY', 'oauth')
